/*****************************************************************************/
/*****************************************************************************/
/**                                                                         **/
/**      IBM SCSI Tape & Medium Changer Device Driver / HP-UX               **/
/**            (C) COPYRIGHT IBM Corporation, 1998, 2007                    **/
/**                                                                         **/
/**          smc.h - IBM SCSI Media Changer Interface Include File          **/
/**                  Licensed Materials - Property of IBM                   **/
/**                           All Rights Reserved                           **/
/**                  Version 1.0.6   10/31/2007                             **/
/**                                                                         **/
/**    US Government Users Restricted Rights  -  Use, duplication, or       **/
/**    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.    **/
/**                                                                         **/
/*****************************************************************************/
/*****************************************************************************/

#ifndef SMC_H
#define SMC_H

#ifndef UCHAR
#define UCHAR
typedef unsigned char   uchar;
#endif

/*===========================================================================*/
/* Basic SCSI IOCTL Command Definitions and Associated Structures            */
/*===========================================================================*/



/*===========================================================================*/
/* SCSI Medium Changer IOCTL Command Definitions and Associated Structures   */
/*===========================================================================*/

/*---------------------------------------------------------------------------*/
/* Defines and data structures for ELEMENT_INFO                              */
/*---------------------------------------------------------------------------*/

#define SMCIOC_ELEMENT_INFO             _IOR('C',3,element_info_t)

typedef struct {
  ushort robot_address;                 /* medium transport element address  */
  ushort robot_count;                   /* number medium transport elements  */
  ushort cell_address;                  /* medium storage element address    */
  ushort cell_count;                    /* number medium storage elements    */
  ushort port_address;                  /* import-export element address     */
  ushort port_count;                    /* number import-export elements     */
  ushort drive_address;                 /* data transfer element address     */
  ushort drive_count;                   /* number data transfer elements     */
} element_info_t;

/*---------------------------------------------------------------------------*/
/* Defines and data structures for MOVE_MEDIUM                               */
/*---------------------------------------------------------------------------*/

#define SMCIOC_MOVE_MEDIUM              _IOW('C',4,move_medium_t)

typedef struct {
  ushort robot;                         /* robot address                     */
  ushort source;                        /* move from location                */
  ushort destination;                   /* move to location                  */
  uchar  invert;                        /* invert medium before insertion    */
} move_medium_t;

#define NO_FLIP         0               /* do not invert medium              */
#define FLIP            1               /* invert medium                     */


/*---------------------------------------------------------------------------*/
/* Defines and data structures for POSITION_TO_ELEMENT                       */
/*---------------------------------------------------------------------------*/

#define SMCIOC_POS_TO_ELEM              _IOW('C',5,pos_to_elem_t)

typedef struct {
  ushort robot;                         /* robot address                     */
  ushort destination;                   /* move to location                  */
  uchar  invert;                        /* invert medium before insertion    */
} pos_to_elem_t;


/*---------------------------------------------------------------------------*/
/* Defines for AUDIT                                                         */
/*---------------------------------------------------------------------------*/

#define SMCIOC_AUDIT                    _IO('C',6)


/*---------------------------------------------------------------------------*/
/* Defines and data structures for INVENTORY                                 */
/*---------------------------------------------------------------------------*/

#define SMCIOC_INVENTORY                _IOW('C',7,inventory_t)

#if defined(__LP64__) && defined (_KERNEL)
#define SMCIOC_INVENTORY32              _IOW('C',7,inventory_t32)
#endif

typedef struct {
  ushort address;                       /* element address                   */
  uchar                 : 2,            /* reserved                          */
        inenab          : 1,            /* medium in robot scope             */
        exenab          : 1,            /* medium not in robot scope         */
        access          : 1,            /* robot access allowed              */
        except          : 1,            /* abnormal element state            */
        impexp          : 1,            /* medium imported or exported       */
        full            : 1;            /* element contains medium           */
  uchar                 : 8;            /* reserved                          */
  uchar asc;                            /* additional sense code             */
  uchar ascq;                           /* additional sense code qualifier   */
  uchar notbus          : 1,            /* element not on same bus as robot  */
                        : 1,            /* reserved                          */
        idvalid         : 1,            /* scsi bus id valid                 */
        luvalid         : 1,            /* logical unit valid                */
                        : 1,            /* reserved                          */
        lun             : 3;            /* logical unit                      */
  uchar scsi;                           /* scsi bus id                       */
  uchar                 : 8;            /* reserved                          */
  uchar svalid          : 1,            /* element address valid             */
        invert          : 1,            /* medium inverted                   */
                        : 6;            /* reserved                          */
  ushort source;                        /* source storage element address    */
  uchar volume[36];                     /* primary volume tag                */
  uchar vendor[80];                     /* vendor specific (padded to 128)   */
} element_status_t;

typedef struct {
  ptr32_t robot_status;       /* medium transport element pages    */
  ptr32_t cell_status;        /* medium storage element pages      */
  ptr32_t port_status;        /* import-export element pages       */
  ptr32_t drive_status;       /* data transfer element pages       */
} inventory_t32;

typedef struct {
  element_status_t *robot_status;       /* medium transport element pages    */
  element_status_t *cell_status;        /* medium storage element pages      */
  element_status_t *port_status;        /* import-export element pages       */
  element_status_t *drive_status;       /* data transfer element pages       */
} inventory_t;


/*---------------------------------------------------------------------------*/
/* Defines for LOCK_DOOR and UNLOCK_DOOR                                     */
/*---------------------------------------------------------------------------*/

#define SMCIOC_LOCK_DOOR                _IOW('C',8,lock_door_t)

typedef uchar lock_door_t;

#define SMC_UNLOCK       0               /* unlock door                   */

#define SMC_LOCK         1               /* lock door                     */


/*---------------------------------------------------------------------------*/
/* Defines and data structures for READ_ELEMENT_DEVIDS (DVCID = 1)           */
/*---------------------------------------------------------------------------*/

#define SMCIOC_READ_ELEMENT_DEVIDS      _IOW('C',9,read_element_devids_t)

#if defined(__LP64__) && defined (_KERNEL)
#define SMCIOC_READ_ELEMENT_DEVIDS32    _IOW('C',9,read_element_devids_t32)
#endif

typedef struct {
    ushort address;             /* element address */
    uchar          :4,          /* reserved */
           access  :1,          /* robot access allowed */
           except  :1,          /* abnormal element state */
                   :1,          /* reserved */
           full    :1;          /* element contains medium */
    uchar  resvd1;              /* reserved */
    uchar  asc;                 /* additional sense code */
    uchar  ascq;                /* additional sense code qualifier */
    uchar  notbus  :1,          /* element not on same bus as robot */
                   :1,          /* reserved */
           idvalid :1,          /* element address valid */
           luvalid :1,          /* logical unit valid */
                   :1,          /* reserved */
           lun     :3;          /* logical unit number */
    uchar  scsi;                /* scsi bus address */
    uchar  resvd2;              /* reserved */
    uchar  svalid  :1,          /* element address valid */
           invert  :1,          /* medium inverted */
                   :6;          /* reserved */
    ushort source;              /* source storage element address */
    uchar          :4,          /* reserved */
           code_set:4;          /* code set X'2' is all ASCII identifier */
    uchar          :4,          /* reserved */
           id_type :4;          /* identifier type */
    uchar  resvd3;              /* reserved */
    uchar  id_len;              /* identifier length */
    uchar  dev_id[36];          /* device identification with serial number */
} element_devid_t;

typedef struct { 
    ushort element_address;               /* starting element address */
    ushort number_elements;               /* number of elements       */
    element_devid_t *drive_devid;         /* data transfer element pages */
} read_element_devids_t;

typedef struct { 
    ushort element_address;               /* starting element address */
    ushort number_elements;               /* number of elements       */
    ptr32_t drive_devid;                  /* data transfer element pages */
} read_element_devids_t32;


/*---------------------------------------------------------------------------*/
/* Defines and data structures for EXCHANGE_MEDIUM                           */
/*---------------------------------------------------------------------------*/

#define SMCIOC_EXCHANGE_MEDIUM          _IOW('C',10,exchange_medium_t)

typedef struct {
    ushort robot;               	/* robot address		 */
    ushort source;              	/* move from location		 */
    ushort destination1;        	/* move to location		 */
    ushort destination2;        	/* move to location		 */
    uchar invert1;              	/* invert before placement into destination 1 */
    uchar invert2;              	/* invert before placement into destination 2 */
} exchange_medium_t;


/*---------------------------------------------------------------------------*/
/* Defines and data structures for INIT_ELEM_STAT_RANGE                      */
/*---------------------------------------------------------------------------*/

#define SMCIOC_INIT_ELEM_STAT_RANGE     _IOW('C',11,element_range_t)

typedef struct {
    ushort element_address;     	/* starting element address	 */
    ushort number_elements;     	/* number of elements      	 */
} element_range_t;

/*---------------------------------------------------------------------------*/
/* Defines and data structures for SMCIOC_READ_CARTRIDGE_LOCATION            */
/*---------------------------------------------------------------------------*/

#define SMCIOC_READ_CARTRIDGE_LOCATION   _IOWR('C',12,read_cartridge_location_t)
#if defined(__LP64__) && defined (_KERNEL)
#define SMCIOC_READ_CARTRIDGE_LOCATION32 _IOWR('C',12,read_cartridge_location_t32)
#endif


typedef struct
{
    ushort address;                            /* element address          */
    uchar  :4,                                 /* reserved                 */
           access:1,                           /* robot access allowed     */
           except:1,                           /* abnormal element state   */
                 :1,                           /* reserved                 */
           full:1;                             /* element contains medium  */
    uchar  resvd1;                             /* reserved                 */
    uchar  asc;                                /* additional sense code    */
    uchar  ascq;                               /* additional sense code qualifier */
    uchar  resvd2[3];                          /* reserved                 */
    uchar  svalid:1,                           /* element address valid    */
           invert:1,                           /* medium inverted          */
           :6;                                 /* reserved                 */
    ushort source;                             /* source storage elem addr */
    uchar  volume[36];                         /* primary volume tag       */
    uchar  :4,                                 /* reserved                 */
           code_set:4;                         /* code set                 */
    uchar  :4,                                 /* reserved                 */
           ident_type:4;                       /* identifier type          */
    uchar  resvd3;                             /* reserved                 */
    uchar  ident_len;                          /* identifier length        */
    uchar  identifier[24];                     /* slot identification      */
} cartridge_location_data_t;

typedef struct
{
    ushort element_address;                    /* starting element address */
    ushort number_elements;                    /* number of elements       */
    cartridge_location_data_t *data;           /* storage element pages    */
    char reserved[8];                          /* reserved                 */
} read_cartridge_location_t;

typedef struct
{
    ushort element_address;                    /* starting element address */
    ushort number_elements;                    /* number of elements       */
    ptr32_t  data;                             /* storage element pages    */
    char reserved[8];                          /* reserved                 */
} read_cartridge_location_t32;

/*---------------------------------------------------------------------------*/

#define SMCIOC_CHANGE_DEFINITION           _IOW('C', 101,int)     

#endif  /* SMC_H */


