/*****************************************************************************/
/*****************************************************************************/
/**                                                                         **/
/**      IBM SCSI Tape & Medium Changer Device Driver / HP-UX               **/
/**            (C) COPYRIGHT IBM Corporation, 1998, 2008                    **/
/**                                                                         **/
/**                 svc.h - Service Utilities Include File                  **/
/**                  Licensed Materials - Property of IBM                   **/
/**                           All Rights Reserved                           **/
/**                  Version 1.0.9   07/16/2008                             **/
/**                                                                         **/
/**    US Government Users Restricted Rights  -  Use, duplication, or       **/
/**    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.    **/
/**                                                                         **/
/*****************************************************************************/
/*****************************************************************************/

#ifndef SVC_H
#define SVC_H

#ifndef UCHAR
#define UCHAR
typedef unsigned char   uchar;
#endif

/*---------------------------------------------------------------------------*/
/* Defines and data structures for DEVICE_SENSE                              */
/*---------------------------------------------------------------------------*/

#define MTDEVICE                        _IOR('m',0x45,device_sn_t)

#define STIOC_DEVICE_SN                 _IOR('T',20,device_sn_t)

typedef uint                            device_sn_t;

/*---------------------------------------------------------------------------*/
/* Defines for FORMAT_TAPE                                                   */
/*---------------------------------------------------------------------------*/

#define STIOC_FORMAT_TAPE               _IO('T',21)

/*---------------------------------------------------------------------------*/
/* Defines for FORCE_DUMP                                                    */
/*---------------------------------------------------------------------------*/

#define STIOC_FORCE_DUMP                _IO('T',22)

/*---------------------------------------------------------------------------*/
/* Defines for STORE_DUMP                                                    */
/*---------------------------------------------------------------------------*/

#define STIOC_STORE_DUMP                _IO('T',23)

/*---------------------------------------------------------------------------*/
/* Defines and data structures for READ/WRITE_BUFFER                         */
/*---------------------------------------------------------------------------*/




typedef struct {
  uchar mode;                                /* transfer mode                    */
  uchar id;                                  /* device buffer id                 */
  uint offset;                               /* buffer offset                    */
  uint size;                                 /* byte count                       */
  uchar *buffer;                             /* data buffer                      */
} buffer_io_t;


typedef struct {
  uchar mode;                                /* transfer mode                    */
  uchar id;                                  /* device buffer id                 */
  uint offset;                               /* buffer offset                    */
  uint size;                                 /* byte count                       */
  ptr32_t buffer;                            /* data buffer                      */
} buffer_io_t32;


#define STIOC_READ_BUFFER               _IOW('T',24,buffer_io_t)
#define STIOC_WRITE_BUFFER              _IOW('T',25,buffer_io_t)
#if defined(__LP64__) && defined(_KERNEL)
#define STIOC_READ_BUFFER32             _IOW('T',24,buffer_io_t32)
#define STIOC_WRITE_BUFFER32            _IOW('T',25,buffer_io_t32)
#endif  /* __LP64__ && _KERNEL */ 



/*
** Read Buffer/Write Buffer mode settings for 3580 Ultrium tape drives.
*/
#define ULT_DESC_DATA_MODE           0x00    /* Read/Write Buffer Descriptor & Data */
#define ULT_DATA_ONLY_MODE           0x02    /* Read/Write Buffer Data only      */
#define ULT_DESC_ONLY_MODE           0x03    /* Read/Write Buffer Descriptor only*/
#define ULT_DWNLD_UCODE_MODE         0x05    /* Download and Save Microcode      */

/*
** Buffer Identifiers for Ultrium tape drives.
** R/O = Read Only;  W/O = Write Only
*/
#define ULT_MAIN_BUF                 0x00    /* Main Data Buffer                 */
#define ULT_DUMP_BUF                 0x01    /* Microcode Dump Buffer, R/O       */
#define ULT_TEST_BUF                 0x02    /* Test Buffer                      */
#define ULT_VPD_BUF                  0x03    /* Vital Product Data buffer        */
#define ULT_UCODE_BUF                0x04    /* Microcode (firmware) buffer, W/O */
#define ULT_CART_BUF                 0x05    /* Cartridge Memory buffer, R/O     */
#define ULT_ERRLOG_BUF               0x06    /* Microcode Dump Buffer, R/O       */

/*
** Read Buffer/Write Buffer mode settings for
** 3581 Ultrium Tape Autoloader
** 3581 ThinStor LTO Gen2 Autoloader
** 3582 Ultrium Scalable Tape Library
** 3572 TS2900  LTO Gen3/4 Autoloader
** 3573 Ultrium Scalable Tape Library
** 3583 Ultrium Scalable Tape Library
** 3576 Ultrium Scalable Tape Library
** 3584 Ultrium UltraScalable Tape Library
** 3577 TS3400 Tape Library
** (For the SCSI Medium Changer only; use 3580 settings for tape drive.)
*/
#define ULT3581_DATA_ONLY_MODE       0x02    /* Read Buffer Data only            */
#define ULT3581_DESC_ONLY_MODE       0x03    /* Read Buffer Descriptor only      */
#define ULT3581_DWNLD_UCODE_MODE     0x05    /* Download and Save Microcode      */

#define NY3581_DATA_ONLY_MODE        0x02    /* Read/Write Buffer Data only      */
#define NY3581_DESC_ONLY_MODE        0x03    /* Read Buffer Descriptor only      */
#define NY3581_DWNLD_UCODE_MODE      0x05    /* Download and Save Microcode      */

#define ULT3582_DATA_ONLY_MODE       0x02    /* Read/Write Buffer Data only      */
#define ULT3582_DESC_ONLY_MODE       0x03    /* Read/Write Buffer Descriptor only*/
#define ULT3582_DWNLD_UCODE_MODE     0x05    /* Download and Save Microcode      */

#define ULT3572_DATA_ONLY_MODE       0x02    /* Read/Write Buffer Data only      */
#define ULT3572_DWNLD_UCODE_MODE     0x05    /* Download and Save Microcode      */

#define ULT3573_DATA_ONLY_MODE       0x02    /* Read/Write Buffer Data only      */
#define ULT3573_DESC_ONLY_MODE       0x03    /* Read/Write Buffer Descriptor only*/
#define ULT3573_DWNLD_UCODE_MODE     0x04    /* Download and Save Microcode      */

#define ULT3583_DATA_ONLY_MODE       0x02    /* Read/Write Buffer Data only      */
#define ULT3583_DESC_ONLY_MODE       0x03    /* Read/Write Buffer Descriptor only*/
#define ULT3583_DWNLD_UCODE_MODE     0x05    /* Download and Save Microcode      */

#define ULT3576_DATA_ONLY_MODE       0x02    /* Read/Write Buffer Data only      */
#define ULT3576_DESC_ONLY_MODE       0x03    /* Read/Write Buffer Descriptor only*/
#define ULT3576_DWNLD_UCODE_MODE     0x05    /* Download and Save Microcode      */

#define ULT3584_DATA_ONLY_MODE       0x01    /* Read Buffer Data only (Vendor)   */
#define ULT3584_DESC_ONLY_MODE       0x03    /* Read Buffer Descriptor only      */
#define ULT3584_DWNLD_UCODE_MODE     0x07    /* Download & Save Ucode w/ offsets */

#define LIB3577_DATA_ONLY_MODE       0x02    /* Write Buffer Data only           */
#define LIB3577_DWNLD_UCODE_MODE     0x05    /* Download & Save Ucode w/ offsets */

/*
** Buffer Identifiers for
** 3581 Ultrium Tape Autoloader
** 3581 ThinStor LTO Gen2 Autoloader
** 3582 Ultrium Scalable Tape Library
** 3572 TS2900  LTO Gen3/4 Autoloader
** 3573 Ultrium Scalable Tape Library
** 3583 Ultrium Scalable Tape Library
** 3576 Ultrium Scalable Tape Library
** 3584 Ultrium UltraScalable Tape Library
** 3577 TS3400 Tape Library
** (For the SCSI Medium Changer only; use 3580 settings for tape drive.)
** R/O = Read Only;  W/O = Write Only
*/
#define ULT3581_UCODE_BUF        0x00    /* Microcode (firmware) Buffer, W/O    */
#define ULT3581_DUMP_BUF         0x01    /* Microcode Dump Buffer, R/O          */

#define NY3581_UCODE_BUF         0x88    /* Microcode (firmware) Buffer, W/O    */
#define NY3581_DUMP_BUF          0x00    /* Microcode Dump Buffer, R/O          */

#define ULT3582_FAIL_BUF         0x00    /* Failure Buffer, R/O                 */
#define ULT3582_UCODE_BUF        0x01    /* Microcode Buffer, R/W               */
#define ULT3582_PRECMD_BUF       0x08    /* Previous Command Log Buffer, R/O    */
#define ULT3582_CMD_BUF          0x09    /* Current Command Log Buffer, R/O     */
#define ULT3582_DUMP_BUF         0x0A    /* Error Log Buffer, R/O               */

#define ULT3572_UCODE_BUF        0x00    /* Microcode (firmware) Buffer, W/O    */

#define ULT3573_FAIL_BUF         0x00    /* Failure Buffer, R/O                 */
#define ULT3573_UCODE_BUF        0x00    /* Microcode Buffer, R/W               */
#define ULT3573_PRECMD_BUF       0x08    /* Previous Command Log Buffer, R/O    */
#define ULT3573_CMD_BUF          0x09    /* Current Command Log Buffer, R/O     */
#define ULT3573_DUMP_BUF         0x0A    /* Error Log Buffer, R/O               */

#define ULT3583_UCODE_BUF        0x00    /* Microcode Buffer, R/W               */
#define ULT3583_CMD_BUF          0x09    /* Command Log Buffer, R/O             */
#define ULT3583_DUMP_BUF         0x0A    /* Error Log Buffer, R/O               */

#define ULT3576_UCODE_BUF        0x00    /* Microcode Buffer, R/W               */
#define ULT3576_CMD_BUF          0x09    /* Command Log Buffer, R/O             */
#define ULT3576_DUMP_BUF         0x0A    /* Error Log Buffer, R/O               */

#define ULT3584_UCODE_BUF        0x00    /* Microcode Buffer, W/O               */
#define ULT3584_DUMP_BUF         0x01    /* Microcode Dump Buffer, R/O          */
#define ULT3584_MOTN_BUF         0x03    /* Motion Buffer, R/O                  */
#define ULT3584_SRVO_BUF         0x04    /* Servo Buffer, R/O                   */
#define ULT3584_STAT_BUF         0x05    /* Statistics Buffer, R/O              */
#define ULT3584_XYC_BUF          0x11    /* XYC Event Buffer, R/O               */
#define ULT3584_OPC_BUF          0x21    /* OPC Event Buffer, R/O               */
#define ULT3584_ACC_BUF          0x31    /* ACC Event Buffer, R/O               */
#define ULT3584_MCC1_BUF         0x41    /* MCC Event Buffer, R/O               */
#define ULT3584_MCC2_BUF         0x42    /* MCC Event Buffer, R/O               */
#define ULT3584_MCC3_BUF         0x43    /* MCC Event Buffer, R/O               */
#define ULT3584_MCC4_BUF         0x44    /* MCC Event Buffer, R/O               */
#define ULT3584_MCC5_BUF         0x45    /* MCC Event Buffer, R/O               */
#define ULT3584_MCC6_BUF         0x46    /* MCC Event Buffer, R/O               */

#define LIB3577_UCODE_BUF        0x00    /* Microcode Buffer, R/W               */

/* Read Buffer/Write Buffer mode settings for Magstar family drives. */
#define MAG_DATA_ONLY_MODE       0x01    /* Read/Write Buffer Data Only         */
#define MAG_DESC_ONLY_MODE       0x03    /* Read/Write Buffer Descriptor Only   */
#define MAG_DWNLD_UCODE_MODE     0x05    /* Download and Save Microcode         */

/*
** Buffer Identifiers for Magstar family drives.
** For all the following, the buffer offset must be on 16-byte
** (4 @ 4-byte words) boundary.
** R/O = Read Only;  W/O = Write Only
*/
#define MAG_DUMP_BUF             0x00    /* Dump and Error Buffer               */
#define MAG_UCODE_BUF            0x01    /* Microcode (Firmware) Buffer         */
#define MAG_WRITE_VPD_BUF        0x02    /* Vital Product Data Write Buffer     */
#define MAG_DRAM_VPD_BUF         0x07    /* DRAM Vital Product Data Buffer      */
#define MAG_EEPROM_VPD_BUF       0x08    /* EEPROM VPD Buffer                   */
#define MAG_DUMP2_BUF            0x09    /* Dump Data #2 Buffer                 */
#define MAG_TEST_BUF             0x0A    /* Test Buffer                         */

#define MAG_SERIAL_BUF           0x0F    /* Serial Number and Checksum  Buffer  */
                                         /* ID, W/O; buffer offset must be 0.   */

/* Kept for downward compatibility only.  Use the new definitions above. */
#define VEND_MODE                0x01    /* vendor specific mode                */
#define DSCR_MODE                0x03    /* descriptor mode                     */
#define DNLD_MODE                0x05    /* download mode                       */

#define ERROR_ID                 0x00    /* error buffer ID                     */
#define UCODE_ID                 0x01    /* ucode buffer ID                     */

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Defines for alternate path support                                        */
/*---------------------------------------------------------------------------*/
typedef struct scsi_path_type
{
  char primary_name[15];              /* primary logical device name                 */
  char primary_parent[15];            /* primary SCSI parent name, "Host" name       */
  uchar primary_id;                   /* primary target address of device, "Id" value*/
  uchar primary_lun;                  /* primary logical unit of device, "lun" value */
  uchar primary_bus;                  /* primary SCSI bus for device, "Channel" value*/
  unsigned long long primary_fcp_scsi_id;  /* primary FCP SCSI id of device          */
  unsigned long long primary_fcp_lun_id;   /* primary FCP logical unit of device     */
  unsigned long long primary_fcp_ww_name;  /* primary FCP world wide name            */
  uchar primary_enabled;              /* primary path enabled                        */
  uchar primary_id_valid;             /* primary id/lun/bus fields valid             */
  uchar primary_fcp_id_valid;         /* primary FCP scsi/lun/id fields              */
  uchar alternate_configured;         /* alternate path configured                   */
  char alternate_name[15];            /* alternate logical device name               */
  char alternate_parent[15];          /* alternate SCSI parent name                  */
  uchar alternate_id;                 /* alternate target address of device          */
  uchar alternate_lun;                /* alternate logical unit of device            */
  uchar alternate_bus;                /* alternate SCSI bus for device               */
  unsigned long long alternate_fcp_scsi_id; /* alternate FCP SCSI id of device       */
  unsigned long long alternate_fcp_lun_id;  /* alternate FCP logical unit of device  */
  unsigned long long alternate_fcp_ww_name; /* alternate FCP world wide name         */
  uchar alternate_enabled;            /* alternate path enabled                      */
  uchar alternate_id_valid;           /* alternate id/lun/bus fields valid           */
  uchar alternate_fcp_id_valid;       /* alternate FCP scsi/lun/id fields            */
  uchar primary_drive_port_valid;     /* primary drive port field valid              */
  uchar primary_drive_port;           /* primary drive port number                   */
  uchar alternate_drive_port_valid;   /* alternate drive port field valid            */
  uchar alternate_drive_port;         /* alternate drive port                        */
  char persistent_dsf[30];            /* persistent logical device name on 11i v3    */
  char reserved[30];
} scsi_path_t;



typedef struct device_path_type
{
  char name[30];                     /* logical device name                         */
  char parent[30];                   /* logical parent name                         */
  uchar id_valid;                    /* SCSI id/lun/bus fields valid                */
  uchar id;                          /* SCSI target address of device               */
  uchar lun;                         /* SCSI logical unit of device                 */
  uchar bus;                         /* SCSI bus for device                         */
  uchar fcp_id_valid;                /* FCP scsi/lun/id fields valid                */
  unsigned long long fcp_scsi_id;    /* FCP SCSi id of device                       */
  unsigned long long fcp_lun_id;     /* FCP logical unit of device                  */
  unsigned long long fcp_ww_name;    /* FCP world wide name                         */
  uchar enabled;                     /* path enabled                                */
  uchar drive_port_valid;            /* drive port field valid                      */
  uchar drive_port;                  /* drive port number                           */
  uchar fenced;                      /* path fenced by disable path ioctl           */
  uchar host;                        /* host bus adapter id                         */
  char persistent_dsf[30];           /* persistent logical device name on 11i v3    */
  char reserved[32];
} device_path_t;

#define MAX_SCSI_FAILOVER_PATH_DISPLAY 16

/* structure to return a bunch of path data about a single device path*/
typedef struct device_paths
{
  int  number_paths;                    /* number of paths configured        */
  int  cur_path;                        /* current active path               */
  device_path_t device_path[MAX_SCSI_FAILOVER_PATH_DISPLAY];
} device_paths_t;

typedef struct add_path
{
  int  NewAltPath;
  char PrimaryMadeUpName[64];
  char SecondaryMadeUpName[64];
} add_path_t;


#define STIOC_QUERY_PATH                _IOR('T',26,scsi_path_t)
#define STIOC_DEVICE_PATH               _IOR('T',27,device_paths_t)
#define STIOC_ENABLE_PATH               _IOW('T',28,dev_t)
#define STIOC_DISABLE_PATH              _IOW('T',29,dev_t)
#define STIOC_ADD_PATH                  _IOW('T',30,add_path_t)
#define STIOC_REMOVE_PATH               _IO('T',31)
#define STIOC_FORCE_FAILOVER            _IO('T',32)

#endif  /* SVC_H */


